function figure4
%% FIGURE4 Produces figure 4 in [1].
% 
%     [1] "Matrix-free Interior Point Method for Compressed Sensing Problems",
%          Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich,
%          ERGO Technical Report, 2012.
%
%   To reproduce figure 4 run:
%              
%     >> figure4
%
% Copyright 2013, Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich

warning('OFF');

% Fix seed.
randn('state',200);
rand ('state',200);

% Size of the signal.
n = 1000;

% Optimal phase transition, obtained from: 
% http://ecos.maths.ed.ac.uk/polytopes.shtml.
o_nus = [0.18942925353957504, 0.24330086659339106, 0.29078428260971806 ,...
         0.3373259117349383 , 0.38568960065819324, 0.4383904866243448  ,...
         0.49884271407221975, 0.5733132922764422 , 0.6781693712361242];

% Allocate memory for mfipm phase transition.
n_nus = zeros(9,1);

% Loop over the measurements.
for m=100:100:900,
  do_ = floor(o_nus(m/100)*m);
    
% Loop over the sparsity.
  for k=do_-2:1:m,
    succs=0;

%   Loop over the trials.
    for trials=1:1:100,      

      % Create the pDCT sensing matrix. Here we assume that the sparsity 
      % matrix is the identity.
      picks = randperm(n);
      A = A_operator(@(x) pdct(x,1,n,sort(picks(1:m))),...
                     @(x) pdct(x,2,n,sort(picks(1:m)))); 

      % Create an exact sparse representation.
      p     = randperm(n);
      x_opt = zeros(n,1); 
      x_opt(p(1:k)) = randn(k,1);

      % Calculate the measurements.
      b = A*x_opt;

      % Set the regularization parameter.
      tau = 1.0e-7;

      % Options for mfipm.
      opts.tol       = 1.0e-15;
      opts.maxiters  = 100;
      opts.tolpcg    = 1.0e-6;
      opts.mxiterpcg = 200;
      opts.verbose   = 0;

      % Call the mfipm solver.
      [x out] = mfipm(n, A, b, tau, opts);
            
      % Project the reconstructed solution.
      proj_x = zeros(length(x),1);
      proj_x(p(1:k)) = x(p(1:k));
    
      rel_err = norm(x_opt-proj_x,2)/norm(x_opt,2);
      if (rel_err <= 1.0e-5), succs = succs+1; end;
    
      fprintf('m=%d,  k=%d,  trial=%d,  r.e=%1.1e,  iters=%d  \n',m,k,trials,rel_err,out.iters);
    
      clearvars -except succs n m k o_nus n_nus trials;

    end;  

    fprintf('Successfull reconstruction percentage: %f \n',succs/trials);
    if succs/trials < 0.5, n_nus(m/100) = (k-1)/m; break; end;
    succs=0;

  end;
end;

% Print the phase transition graph.
rhos = (100:100:900)'./1000;

h = figure(1);

plot(rhos, o_nus, 'black', 'LineWidth', 3);
hold on;
plot(rhos, n_nus, 'green--', 'LineWidth', 3);

leg = legend('Optimal', 'mfipm', 'Location', 'SouthEast');
set(leg,'FontSize',11);

xlabel('\bf{\rho=m/n}', 'FontSize', 14);
text = '\bf{Phase transition for matrix-free IPM}';
ylabel('\bf{\nu=k/m}', 'FontSize', 14, 'Rotation', 90);
title(text, 'FontSize', 14);

grid on;

print('-dpdf','-zbuffer','-r200','figure4');

clear;

end % figure4
