function table5
%% TABLE5 Repeats all the experiments in table 5 of [1].
% 
%   [1] "Matrix-free Interior Point Method for Compressed Sensing Problems",
%        Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich,
%        ERGO Technical Report, 2012.
%
% To repeat the experiments in table 5 run:
%              
%   >> table5
%
% Some additional information to table 5 are printed at the MATLAB prompt.
%
% Explanation of the output at the MATLAB prompt:
%
% Itr:     Iterations needed for reconstruction.
% xo:      Optimal sparse representation.
% x:       Reconstructed projected representation. Please see [1] for
%          details regarding the projection of the reconstructed solution.
% e:       White gaussian noise in vector form, e\in R^{m}.
% nMat:    Number of matrix-vector products needed.
% Rel Res: Rellative residual ||Ax-b||/||b||.
%
% || ||:   The Euclidean norm.
%
% || ||_1: The ell_1 norm.
%
%
%  WARNING!!! This task is computationally demanding.
%
% Copyright 2013, Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich
%% 

warning('OFF');

% Sparco problems (ID).
sparco_probs = [2, 3, 5, 6, 7, 9, 10, 11, 401, 402, 403, 601, 602, 603 ,... 
                701, 702, 902, 903];
            
% L1-regularization parameters for noiseless measurements.
taus   = [1.0e-10,  1.0e-7, 1.0e-4, 1.0e-3, 1.0e-7, 1.0e-10, 1.0e-10   ,... 
           1.0e-2,  1.0e-8, 1.0e-8, 1.0e-4, 1.0e-4,  1.0e-4,  1.0e-8   ,... 
          1.0e-10, 1.0e-10,  1.0e-7,  1.0e-5];

% Parameters for mfipm.
tolmfipm_n = [ 1.0e-8, 1.0e-6,  1.0e-8, 1.0e-6, 1.0e-10, 1.0e-6, 1.0e-8,... 
               1.0e-6, 1.0e-8,  1.0e-8, 1.0e-6,  1.0e-6, 1.0e-6, 1.0e-8,...
               1.0e-8, 1.0e-8, 1.0e-10, 1.0e-8];

tolmfipm   = [1.0e-12, 1.0e-14, 1.0e-12,  1.0e-7, 1.0e-14,  1.0e-8     ,...
              1.0e-10,  1.0e-8, 1.0e-12, 1.0e-12, 1.0e-10,  1.0e-8     ,... 
               1.0e-8, 1.0e-10, 1.0e-12, 1.0e-10, 1.0e-14, 1.0e-12]; 

tolpcgmfipm_n = [1.0e-1, 1.0e-2, 1.0e-2, 1.0e-2, 1.0e-2, 1.0e-1, 1.0e-2,... 
                 1.0e-1, 1.0e-2, 1.0e-2, 1.0e-2, 1.0e-1, 1.0e-1, 1.0e-2,... 
                 1.0e-1, 1.0e-1, 1.0e-2, 1.0e-2];

tolpcgmfipm   = [1.0e-1, 1.0e-2, 1.0e-2, 1.0e-6, 1.0e-2, 1.0e-2, 1.0e-2,... 
                 1.0e-2, 1.0e-2, 1.0e-4, 1.0e-4, 1.0e-2, 1.0e-4, 1.0e-4,... 
                 1.0e-1, 1.0e-2, 1.0e-2, 1.0e-2];

% SNR measured in dB. This fixed for all experiments in table 4.
snr = 60.0;
  
% Use the termination criterion for solvers which depends on the rellative 
% error and the magnitude of the l1-norm of the recovered solution obtained 
% by mfipm.
crit_mfipm = 1;
  
% Turn on diary for backup.
diary('table5');
diary on;

%% Loop over the Sparco problems.
for prob = [1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18]
      
  fprintf('SPARCO PROBLEM %d \n \n', sparco_probs(prob));
    
  % Fix seed for each problem.
  randn('state',200);
  rand ('state',200);
  
  %% Initialize problem.
  % Generate the problem and initialize the data. 
  P = generateProblem(sparco_probs(prob));
  
  n = P.sizeA(2);
  m = P.sizeA(1);
  
  Af  = @(x) P.A(x, 1);
  AfT = @(y) P.A(y, 2);
  A   = A_operator(@(x) Af(x), @(x) AfT(x));

  b = P.b;
    
  % Add noise.
  b_n = awgn(b, snr, 'measured');
  n2e = norm(b - b_n);
  
  %% Load xo.
  %  The optimal solutions for problems 401, 402, 403, 601, 602, 603, 701,
  %  702, have been generated using the SPGL1 solver by
  % setting its termination criteria parameters to
  % optionsspg.bpTol       = 1e-15;
  % optionsspg.optTol      = 1e-15; 
  % optionsspg.dect_ipmsol = 1e-15; 
  % optionsspg.maxMatvec   = 40000;
  % optionsspg.iterations  = 20000;
  %
  if sparco_probs(prob) == 401
    load xo401.mat;
    prj = 1; % Project the optimal solution for Sparco problem 401.
  elseif sparco_probs(prob) == 402
    load xo402.mat;
    prj = 1; % Project the optimal solution for Sparco problem 402.
  elseif sparco_probs(prob) == 403
    load xo403.mat;
    prj = 1; % Project the optimal solution for Sparco problem 403.
  elseif sparco_probs(prob) == 601
    load xo601.mat;
    prj = 1; % Project the optimal solution for Sparco problem 601.
  elseif sparco_probs(prob) == 602 
    load xo602.mat;
    prj = 1; % Project the optimal solution for Sparco problem 602.
  elseif sparco_probs(prob) == 603 
    load xo603.mat;
    prj = 1; % Project the optimal solution for Sparco problem 603.
  elseif sparco_probs(prob) == 701 
    load xo701.mat;
    prj = 1; % Project the optimal solution for Sparco problem 701.
  elseif sparco_probs(prob) == 702
    load xo702.mat;
    prj = 1; % Project the optimal solution for Sparco problem 702.
  else
    xo = P.x0;
    prj = 0;
  end;
  
  %% Project the xo for Sparco problems 401, 701, 702.
  if prj  
    % Find the nonzeros for xo.
    norm1xo099        = 0.999*norm(xo,1);
    [srt_xo srt_idxo] = sort(abs(xo),'descend');
    sum_srt_xo        = 0.0;
      
    for i=1:length(srt_xo), 
      sum_srt_xo = sum_srt_xo + srt_xo(i); 
      if(sum_srt_xo >= norm1xo099) 
        nnzeros = i; 
        break; 
      end
    end
      
    % Project xo.
    zrSrtIdxXo     = sort(srt_idxo(nnzeros+1:end));
    xo(zrSrtIdxXo) = 0.0;
  end

  %% Load the optimal tau, which makes BP_{epsilon_2} and BPDN equivalent.
  %  These taus have been obtained using the SPGL1 solver by
  %  setting its termination criteria parameters to
  %  optionsspg.bpTol       = 1e-15;
  %  optionsspg.optTol      = 1e-15; 
  %  optionsspg.dect_ipmsol = 1e-15; 
  %  optionsspg.maxMatvec   = 40000;
  %  optionsspg.iterations  = 20000;
  %
  if sparco_probs(prob) == 2
    load opttaun2.mat;
  elseif sparco_probs(prob) == 3 
    load opttaun3.mat;
  elseif sparco_probs(prob) == 5
    load opttaun5.mat;
  elseif sparco_probs(prob) == 6 
    load opttaun6.mat;
  elseif sparco_probs(prob) == 7
    load opttaun7.mat;
  elseif sparco_probs(prob) == 9
    load opttaun9.mat;
  elseif sparco_probs(prob) == 10
    load opttaun10.mat;
  elseif sparco_probs(prob) == 11
    load opttaun11.mat;
  elseif sparco_probs(prob) == 401
    load opttaun401.mat;
  elseif sparco_probs(prob) == 402
    load opttaun402.mat;
  elseif sparco_probs(prob) == 403
    load opttaun403.mat;
  elseif sparco_probs(prob) == 601
    load opttaun601.mat;
  elseif sparco_probs(prob) == 602
    load opttaun602.mat;
  elseif sparco_probs(prob) == 603
    load opttaun603.mat;
  elseif sparco_probs(prob) == 701
    load opttaun701.mat;
  elseif sparco_probs(prob) == 702
    load opttaun702.mat;
  elseif sparco_probs(prob) == 902
    load opttaun902.mat;
  elseif sparco_probs(prob) == 903
    load opttaun903.mat;
  end;

  %% Here for noisy measurements & mfipm.
  disp('MFIPM NOISY');
    
  opts2.tol       = tolmfipm_n(prob);
  opts2.tolpcg    = tolpcgmfipm_n(prob);
  opts2.maxiters  = 100;
  opts2.mxiterpcg = 200;
  opts2.verbose   = 0;

  [x out] = mfipm(n, A, b_n, opttaun, opts2);
      
  % Measure the quality of the reconstructed representation.
  [re_n_mfipm, n1diff_n_mfipm, obj_n_mfipm, resdiff_n_mfipm] = ...
     mqinterface(b_n,A,x,xo,n2e,out.iters,out.nMat,out.totcputime,opttaun);
    
  %% Here for noiseless measurements & mfipm.
  disp('MFIPM NOISELESS');

  opts2.tol       = tolmfipm(prob);
  opts2.tolpcg    = tolpcgmfipm(prob);
  opts2.maxiters  = 100;
  opts2.mxiterpcg = 200;
  opts2.verbose   = 0;

  [x out] = mfipm(n,A,b,taus(prob),opts2);
    
  % Measure the quality of the reconstructed representation.
  [re_mfipm, n1diff_mfipm, obj_mfipm, resdiff_mfipm] = ...
      mqinterface(b,A,x,xo,0,out.iters,out.nMat,out.totcputime,taus(prob));
  
  %% Here for noisy measurements & FPC_AS.
  disp('FPC NOISY');
  opts.mxitr        = 20000;
  opts.gtol         = 1.0e-15;
  opts.gtol_scale_x = 1.0e-15;
  opts.PrintOptions = 0;  
  opts.record       = -1;
    
  [x, Out] = ...
   FPC_AS(n,A,b_n,opttaun,[],opts,xo,re_n_mfipm,[],obj_n_mfipm,crit_mfipm);
    
  % Measure the quality of the reconstructed representation.
  mqinterface(b_n,A,x,xo,n2e,Out.itr,Out.nProdA+Out.nProdAt,Out.cpu,0);
    
  %% Here for noiseless measurements & FPC_AS.
  disp('FPC NOISELESS');
  opts.mxitr        = 20000;
  opts.gtol         = 1.0e-15;
  opts.gtol_scale_x = 1.0e-15;
  opts.PrintOptions = 0;  
  opts.record       = -1;
    
  [x, Out] = ...
      FPC_AS(n,A,b,taus(prob),[],opts,xo,re_mfipm,[],obj_mfipm,crit_mfipm);
    
  % Measure the quality of the reconstructed representation.
  mqinterface(b,A,x,xo,0,Out.itr,Out.nProdA+Out.nProdAt,Out.cpu,0);

  %% Here for noisy measurements & SPGL1_bpdn.
  disp('SPGL1 NOISY');
  options.bpTol       = 1e-15;
  options.optTol      = 1e-15;
  options.dect_ipmsol = 1e-15; 
  options.maxMatvec   = 40000;
  options.iterations  = 20000;
  options.verbosity   = -1;
    
  [x,~,~,Out] = ...
      spg_bpdn(A,b_n,n2e,options,xo,re_n_mfipm,n1diff_n_mfipm,...
      obj_n_mfipm,resdiff_n_mfipm,opttaun,crit_mfipm);
    
  % Measure the quality of the reconstructed representation.
  mqinterface(b_n,A,x,xo,n2e,Out.iter,Out.nProdA+Out.nProdAt,...
      Out.timeTotal,0);
    
  %% Here for noiseless measurements & SPGL1_bp.
  disp('SPGL1 NOISELESS');
  options.bpTol       = 1e-15;
  options.optTol      = 1e-15; 
  options.dect_ipmsol = 1e-15; 
  options.maxMatvec   = 40000;
  options.iterations  = 20000;
  options.verbosity   = -1;
    
  [x,~,~,Out] = ...
      spg_bp(A,b,options,xo,re_mfipm,n1diff_mfipm,obj_mfipm,...
      resdiff_mfipm,taus(prob),crit_mfipm);
    
  % Measure the quality of the reconstructed representation.
  mqinterface(b,A,x,xo,0,Out.iter,Out.nProdA+Out.nProdAt,Out.timeTotal,0);

  %% Here for noisy measurements & l1_ls.
  disp('L1 LS NOISY');
    
  tols = 1.0e-15; 
  
  tstart = cputime; 
  [x, ~, ~, nProdA, nProdAt, itr] = ...
        l1_ls(A,xo,re_n_mfipm,[],obj_n_mfipm,crit_mfipm,A',m,n,b_n,...
        opttaun,tols,true);
  tend = cputime;
  l1lscpu = tend - tstart;
  
  % Measure the quality of the reconstructed representation.
  mqinterface(b_n,A,x,xo,n2e,itr,nProdA+nProdAt,l1lscpu,0);
    
  %% Here for noiseless measurements & l1_ls.
  disp('L1 LS NOISELESS');
    
  tols = 1.0e-15; 

  tstart = cputime; 
  [x, ~, ~, nProdA, nProdAt, itr] = ...
           l1_ls(A,xo,re_mfipm,[],obj_mfipm,crit_mfipm,A',m,n,b,...
           taus(prob),tols,true);
  tend = cputime;
  l1lscpu = tend - tstart;
  
  % Measure the quality of the reconstructed representation.
  mqinterface(b,A,x,xo,0,itr,nProdA+nProdAt,l1lscpu,0);
    
  %% Here for noisy measurements & PDCO.
  disp('PDCO NOISY');
  
  OptTol   = 1.0e-15;
  maxIters = 1000;
  
  [x, nProdA, nProdAt, itr, pdcotime] = ...
     SolveBP(A,b_n,n,maxIters,opttaun,OptTol,xo,re_n_mfipm,[],obj_n_mfipm);
    
  disp(' ');
  % Measure the quality of the reconstructed representation.
  mqinterface(b_n,A,x,xo,n2e,itr,nProdA+nProdAt,pdcotime,0);
    
  %% Here for noisy measurements & PDCO.
  disp('PDCO NOISELESS');
  
  OptTol   = 1.0e-15;
  maxIters = 1000;
  
  [x, nProdA, nProdAt, itr, pdcotime] = ...
        SolveBP(A,b,n,maxIters,taus(prob),OptTol,xo,re_mfipm,[],obj_mfipm);
    
  disp(' ');
  % Measure the quality of the reconstructed representation.
  mqinterface(b,A,x,xo,0,itr,nProdA+nProdAt,pdcotime,0);
    
  fprintf('\n \n');
  diary off;
  diary on;
end

diary off;
end % table5
