function table7
%% TABLE789 Produces table 7 in [1].
% 
%     [1] "Matrix-free Interior Point Method for Compressed Sensing Problems",
%          Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich,
%          ERGO Technical Report, 2012.
%
%   To produce table 7 run:
%              
%     >> table7
%
%  The rellative error of the solutions is also printed.
%
%  WARNING!!! This task is computationally demanding.
%
% Copyright 2013, Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich

warning('OFF');

%% Initialize stats options and problems.
%  Sparco problems (ID).
sparco_probs = [2, 3, 5, 6, 7, 9, 10, 11, 401, 402, 403, 601, 602, 603 ,... 
                701, 702, 902, 903];

% Initialize SNR.
snrs = 10:10:120;
   
% mfipm parameters.
opts2.tol       = 1.0e-15;
opts2.maxiters  = 100;
opts2.tolpcg    = 1.0e-10;
opts2.mxiterpcg = 1000;
opts2.verbose   = 0;

% l1_ls parameters.
l1_lsTol = 1.0e-15;

% Parameters for SPGL1.
optionsspg.bpTol       = 1e-15;
optionsspg.optTol      = 1e-15; 
optionsspg.dect_ipmsol = 1e-15; 
optionsspg.maxMatvec   = 40000;
optionsspg.iterations  = 20000;
optionsspg.verbosity   = -1;

% Parameters for FPC.
opts_fpc.gtol         = 1e-15;
opts_fpc.gtol_scale_x = 1e-15;
opts_fpc.mxitr        = 4000; 
opts_fpc.record       = -1;

% Memory allocation for stats.
amp_vec_mfipm = zeros(length(snrs),1);
amp_vec_opt   = zeros(length(snrs),1);
amp_vec_fpc   = zeros(length(snrs),1);
amp_vec_l1_ls = zeros(length(snrs),1);

re_vec_mfipm = zeros(length(snrs),1);
re_vec_fpc   = zeros(length(snrs),1);
re_vec_l1_ls = zeros(length(snrs),1);

% Controls the number of trials performed
% per SNR per problem. The suggested value
% is 1, more than that it will take way too
% much time for the process to be finished.
trialsmax = 1;

diary('table7');
diary on;

%% Start the loop on the problems.
for prob = 1:length(sparco_probs)
  
  % Fix the seed.
  randn('state', 200);
  rand ('state', 200);
  
  %% Generate the problem and initialize the data.
  P = generateProblem(sparco_probs(prob));

  n  = P.sizeA(2);
  m  = P.sizeA(1);
  
  Af = @(x) P.A(x,1);
  AT = @(y) P.A(y,2);
  A  = A_operator(@(x) Af(x), @(x) AT(x));
  
  %% Load xo.
  %  The optimal solutions for problems 401, 402, 403, 601, 602, 603, 701,
  %  702, have been generated using the SPGL1 solver by
  % setting its termination criteria parameters to
  % optionsspg.bpTol       = 1e-15;
  % optionsspg.optTol      = 1e-15; 
  % optionsspg.dect_ipmsol = 1e-15; 
  % optionsspg.maxMatvec   = 40000;
  % optionsspg.iterations  = 20000;
  %
  if sparco_probs(prob) == 401;
    load xo401.mat;
    prj = 1; % Project the optimal solution for Sparco problem 401.
  elseif sparco_probs(prob) == 402; 
    load xo402.mat;
    prj = 1; % Project the optimal solution for Sparco problem 402.
  elseif sparco_probs(prob) == 403; 
    load xo403.mat;
    prj = 1; % Project the optimal solution for Sparco problem 403.
  elseif sparco_probs(prob) == 601; 
    load xo601.mat;
    prj = 1; % Project the optimal solution for Sparco problem 601.
  elseif sparco_probs(prob) == 602; 
    load xo602.mat;
    prj = 1; % Project the optimal solution for Sparco problem 602.
  elseif sparco_probs(prob) == 603; 
    load xo603.mat;
    prj = 1; % Project the optimal solution for Sparco problem 603.
  elseif sparco_probs(prob) == 701; 
    load xo701.mat;
    prj = 1; % Project the optimal solution for Sparco problem 701.
  elseif sparco_probs(prob) == 702;
    load xo702.mat;
    prj = 1; % Project the optimal solution for Sparco problem 702.
  else
    xo = P.x0;
    prj = 0;
  end;
  
  %% Project xo for Sparco problems 401, 402, 403, 601, 602, 603, 701, 702.
  if prj  
    % Find the nonzeros for xo.
    norm1xo099        = 0.999*norm(xo,1);
    [srt_xo srt_idxo] = sort(abs(xo),'descend');
    sum_srt_xo        = 0.0;
      
    for i=1:length(srt_xo), 
      sum_srt_xo = sum_srt_xo + srt_xo(i); 
      if(sum_srt_xo >= norm1xo099) 
        nnzeros = i; 
        break; 
      end
    end
      
    % Project xo.
    zrSrtIdxXo     = sort(srt_idxo(nnzeros+1:end));
    xo(zrSrtIdxXo) = 0.0;
  end
  
  % Initialize stats for each problem.
  sumsnrmfipm = 0;
  sumsnrfpc   = 0;
  sumsnrl1ls  = 0;
  
  sumsnrmfipm2 = 0;
  sumsnrfpc2   = 0;
  sumsnrl1ls2  = 0;
  
  %% Loop over all snrs.
  for i = 1:length(snrs),
      
    % Initialize stats for each SNR.
    sum_amp_mfipm = 0.0;
    sum_amp_opt   = 0.0;
    sum_amp_fpc   = 0.0;
    sum_amp_l1_ls = 0.0;
    
    sum_re_mfipm = 0.0;
    sum_re_fpc   = 0.0;
    sum_re_l1_ls = 0.0;
    
    sum_accuracymfipm = 0.0;
    sum_accuracyfpc   = 0.0;
    sum_accuracyl1_ls = 0.0;
    
    sum_taus = 0.0;
    
    %% Loop over all trials per snr.
    for trials = 1:1:trialsmax,
  
      % add noise
      b_  = P.b;
      b   = awgn(b_,snrs(i),'measured');
      n2e = norm(b_-b,2);
      
      % Find optimal solution given noise using the SPGL1 solver.
      [x_opt,~,~,info] = spg_bpdn(A,b,n2e,optionsspg,xo,1.0e+5,1.0e+5,...
          1.0e+5,1.0e+5,1.0e+5,0);
           
      idx_nnz_xo             = find(xo~=0);
      temp_x_opt             = zeros(length(x_opt),1);
      temp_x_opt(idx_nnz_xo) = x_opt(idx_nnz_xo);
      n2eX_opt               = norm(xo-temp_x_opt,2)^2;
      amp_opt                = sqrt(n2eX_opt/n)/sqrt(n2e^2/m);
      sum_amp_opt            = sum_amp_opt+amp_opt;
      
      % Call mfipm.
      xmfipm = mfipm(n,A,b,max(info.gNorm,1.0e-9),opts2);
         
      temp_xmfipm             = zeros(length(xmfipm),1);
      temp_xmfipm(idx_nnz_xo) = xmfipm(idx_nnz_xo);
      n2exmfipm               = norm(xo-temp_xmfipm,2)^2;
      amp_mfipm               = sqrt(n2exmfipm/n)/sqrt(n2e^2/m);
      sum_amp_mfipm           = sum_amp_mfipm+amp_mfipm;
      sum_re_mfipm            = sum_re_mfipm+norm(xo-temp_xmfipm)/norm(xo);
      sum_accuracymfipm       = ...
          sum_accuracymfipm+max(amp_mfipm-amp_opt,0)/amp_opt;
      
       % Call l1_ls.
      xl1_ls = ...
        l1_ls(A,xo,1.0e+5,[],1.0e+5,0,A',m,n,b,max(info.gNorm,1.0e-9),...
        l1_lsTol,true);
           
      temp_xl1_ls             = zeros(length(xl1_ls),1);
      temp_xl1_ls(idx_nnz_xo) = xl1_ls(idx_nnz_xo);
      n2exl1_ls               = norm(xo-temp_xl1_ls,2)^2;
      amp_l1_ls               = sqrt(n2exl1_ls/n)/sqrt(n2e^2/m);
      sum_amp_l1_ls           = sum_amp_l1_ls+amp_l1_ls;
      sum_re_l1_ls            = ...
          sum_re_l1_ls+norm(xo-temp_xl1_ls)/norm(xo);
      sum_accuracyl1_ls       = ...
          sum_accuracyl1_ls+max(amp_l1_ls-amp_opt,0)/amp_opt;
      
      % Call FPC_AS.
      xfpc = ...
       FPC_AS(n,A,b,max(info.gNorm,1.0e-9),[],opts_fpc,xo,1.0e+5,[],...
       1.0e+5,0);
           
      temp_xfpc             = zeros(length(xfpc),1);
      temp_xfpc(idx_nnz_xo) = xfpc(idx_nnz_xo);
      n2exfpc               = norm(xo-temp_xfpc,2)^2;
      amp_fpc               = sqrt(n2exfpc/n)/sqrt(n2e^2/m);
      sum_amp_fpc           = sum_amp_fpc+amp_fpc;
      sum_re_fpc            = sum_re_fpc+norm(xo-temp_xfpc)/norm(xo);
      sum_accuracyfpc       = ...
          sum_accuracyfpc+max(amp_fpc-amp_opt,0)/amp_opt;
      
      % Add taus
      sum_taus = sum_taus+max(info.gNorm,1.0e-9);
      
    end
    
    % Compute average stats.
    amp_vec_mfipm(i) = sum_amp_mfipm/trials;
    amp_vec_fpc(i)   = sum_amp_fpc/trials;
    amp_vec_l1_ls(i) = sum_amp_l1_ls/trials;
    amp_vec_opt(i)   = sum_amp_opt/trials;

    re_vec_fpc(i)   = sum_re_fpc/trials;
    re_vec_l1_ls(i) = sum_re_l1_ls/trials;
    re_vec_mfipm(i) = sum_re_mfipm/trials;
    
    avg_tau = sum_taus/trials;
    
    accuracymfipm = sum_accuracymfipm/trials;
    txt = ['Sparco problem %3d, SNR %3d, tau %1.1e,',...
        ' rampd mfipm %11.1e, r.e mfipm %1.1e \n'];
    fprintf(txt,sparco_probs(prob),snrs(i),avg_tau,accuracymfipm,...
        re_vec_mfipm(i));
    accuracyfpc = sum_accuracyfpc/trials;
    txt = ['Sparco problem %3d, SNR %3d, tau %1.1e,',...
        ' rampd fpc   %11.1e, r.e fpc   %1.1e \n'];
    fprintf(txt,sparco_probs(prob),snrs(i),avg_tau,accuracyfpc,...
        re_vec_fpc(i));
    accuracyl1_ls = sum_accuracyl1_ls/trials;
    txt = ['Sparco problem %3d, SNR %3d, tau %1.1e,',...
        ' rampd l1_ls %11.1e, r.e l1_ls %1.1e \n'];
    fprintf(txt,sparco_probs(prob),snrs(i),avg_tau,accuracyl1_ls,...
        re_vec_l1_ls(i));
    
    sumsnrmfipm = sumsnrmfipm + accuracymfipm;
    sumsnrfpc   = sumsnrfpc + accuracyfpc;
    sumsnrl1ls  = sumsnrl1ls + accuracyl1_ls;
    
    if snrs(i) <= 60
       sumsnrmfipm2 = sumsnrmfipm2 + accuracymfipm;
       sumsnrfpc2   = sumsnrfpc2 + accuracyfpc;
       sumsnrl1ls2  = sumsnrl1ls2 + accuracyl1_ls;   
    end

    diary off;
    diary on;
  end
  
  avgsnrmfipm = sumsnrmfipm/length(snrs);
  txt = 'Sparco problem %3d, avg. rampd over all SNRs for mfipm: %1.1e\n';
  fprintf(txt,sparco_probs(prob),avgsnrmfipm);
  avgsnrmfipm2 = 2*sumsnrmfipm2/length(snrs);
  txt = ['Sparco problem %3d, avg. rampd over SNRs from 10 to 60',...
         ' dB for mfipm: %1.1e\n'];
  fprintf(txt,sparco_probs(prob),avgsnrmfipm2);
  avgsnrfpc   = sumsnrfpc/length(snrs);
  txt = 'Sparco problem %3d, avg. rampd over all SNRs for fpc:   %1.1e\n';
  fprintf(txt,sparco_probs(prob),avgsnrfpc);
  avgsnrfpc2 = 2*sumsnrfpc2/length(snrs);
  txt = ['Sparco problem %3d, avg. rampd over SNRs from 10 to 60',...
         ' dB for fpc: %1.1e\n'];
  fprintf(txt,sparco_probs(prob),avgsnrfpc2);
  avgsnrl1ls  = sumsnrl1ls/length(snrs);
  txt = 'Sparco problem %3d, avg. rampd over all SNRs for l1_ls: %1.1e\n';
  fprintf(txt,sparco_probs(prob),avgsnrl1ls);
  avgsnrl1ls2 = 2*sumsnrl1ls2/length(snrs);
  txt = ['Sparco problem %3d, avg. rampd over SNRs from 10 to 60',...
         ' dB for l1_ls: %1.1e'];
  fprintf(txt,sparco_probs(prob),avgsnrl1ls2);
 
  fprintf('\n \n');
 
end

diary off;

end % table7
